import { __assign } from "tslib";
import React, { useState, useRef, useEffect, useMemo } from 'react';
import PropTypes from 'prop-types';
import { PaintPanelS, ButtonS, ToolbarS, ZoomBarS, ColorBarS, ToolbarItemS, RangeInputS, ColorButtonS, SeparatorS, ToolbarHolderS, PDFWrapperS, DrawPanelS, SelectS, OptionS, } from './Paintboard.styled';
import { fabric } from 'fabric-all-modules';
import { PdfReader } from '../PdfReader';
import { saveAs } from 'file-saver';
import { Board, modes } from './Board.Class.js';
import { ColorPicker } from '../ColorPicker';
import SelectIcon from '../images/cross.svg';
import EraserIcon from '../images/eraser.svg';
import PencilIcon from '../images/pencil.svg';
import DeleteIcon from '../images/delete.svg';
import ZoomInIcon from '../images/zoom-in.svg';
import ZoomOutIcon from '../images/zoom-out.svg';
import DownloadIcon from '../images/download.svg';
import UploadIcon from '../images/add-photo.svg';
import FillIcon from '../images/color-fill.svg';
var initFileInfo = {
    file: { name: 'whiteboard' },
    totalPages: 1,
    currentPageNumber: 0,
    currentPage: '',
    saveAsWidth: 512,
    saveAsHeight: 512,
    displayWidth: 512,
    displayHeight: 512
};
var initDrawingSettings = {
    brushWidth: 5,
    currentMode: 'PENCIL',
    currentColor: '#000000',
    brushWidth: 5,
    fill: false,
};
var initSettings = {
    zoom: 1,
    contentJSON: null,
};
var Paintboard = function (_a) {
    var mode = _a.mode, _b = _a.maxWidth, maxWidth = _b === void 0 ? 800 : _b, _c = _a.maxHeight, maxHeight = _c === void 0 ? 800 : _c, sizes = _a.sizes, backgroundImage = _a.backgroundImage, controls = _a.controls, settings = _a.settings, drawingSettings = _a.drawingSettings, fileInfo = _a.fileInfo, _d = _a.onObjectAdded, onObjectAdded = _d === void 0 ? function (data, event, canvas) { } : _d, _e = _a.onObjectRemoved, onObjectRemoved = _e === void 0 ? function (data, event, canvas) { } : _e, _f = _a.onObjectModified, onObjectModified = _f === void 0 ? function (data, event, canvas) { } : _f, _g = _a.onCanvasRender, onCanvasRender = _g === void 0 ? function (data, event, canvas) { } : _g, _h = _a.onCanvasChange, onCanvasChange = _h === void 0 ? function (data, event, canvas) { } : _h, _j = _a.onZoom, onZoom = _j === void 0 ? function (data, event, canvas) { } : _j, _k = _a.onImageUploaded, onImageUploaded = _k === void 0 ? function (data, event, canvas) { } : _k, _l = _a.onPDFUploaded, onPDFUploaded = _l === void 0 ? function (data, event, canvas) { } : _l, _m = _a.onPDFUpdated, onPDFUpdated = _m === void 0 ? function (data, event, canvas) { } : _m, _o = _a.onPageChange, onPageChange = _o === void 0 ? function (data, event, canvas) { } : _o, _p = _a.onOptionsChange, onOptionsChange = _p === void 0 ? function (data, event, canvas) { } : _p, _q = _a.onSaveCanvasAsImage, onSaveCanvasAsImage = _q === void 0 ? function (data, event, canvas) { } : _q, _r = _a.onExportBlob, onExportBlob = _r === void 0 ? function (data, event, canvas) { } : _r, _s = _a.onConfigChange, onConfigChange = _s === void 0 ? function (data, event, canvas) { } : _s, _t = _a.onSaveCanvasState, onSaveCanvasState = _t === void 0 ? function (data, event, canvas) { } : _t;
    var _u = useState(), board = _u[0], setBoard = _u[1];
    var _v = useState({}), canvasObjectsPerPage = _v[0], setCanvasObjectsPerPage = _v[1];
    var _w = useState(__assign(__assign({}, initDrawingSettings), drawingSettings)), canvasDrawingSettings = _w[0], setCanvasDrawingSettings = _w[1];
    var canvasConfig = __assign(__assign({}, initSettings), settings);
    var _x = useState(canvasConfig.zoom), zoom = _x[0], setZoom = _x[1];
    var _y = useState(__assign(__assign({}, initFileInfo), fileInfo)), fileReaderInfo = _y[0], setFileReaderInfo = _y[1];
    var canvasRef = useRef(null);
    var whiteboardRef = useRef(null);
    var uploadPdfRef = useRef(null);
    var rangeInput = useRef(null);
    var enabledControls = useMemo(function () {
        var _a;
        return __assign((_a = {}, _a[modes.PENCIL] = true, _a[modes.MOVE] = true, _a[modes.ERASERDRAW] = true, _a.CLEAR = true, _a.FILL = true, _a.BRUSH = true, _a.COLOR_PICKER = true, _a.DEFAULT_COLORS = true, _a.FILES = true, _a.SAVE_AS_IMAGE = true, _a.SAVE_AS_IMAGE_WITHOUT_BACKGROUNDIMAGE = true, _a.EXPORT_IMAGE = true, _a.EXPORT_IMAGE_WITHOUT_BACKGROUNDIMAGE = true, _a.ZOOM = true, _a), controls);
    }, [controls]);
    useEffect(function () {
        setCanvasDrawingSettings(__assign(__assign({}, canvasDrawingSettings), drawingSettings));
    }, [drawingSettings]);
    useEffect(function () {
        if (!board || !canvasConfig)
            return;
        board.setCanvasConfig(canvasConfig);
    }, [settings]);
    useEffect(function () {
        setFileReaderInfo(__assign(__assign({}, fileReaderInfo), fileInfo));
    }, [fileInfo]);
    useEffect(function () {
        if (board) {
            return;
        }
        var newBoard = new Board({
            drawingSettings: canvasDrawingSettings,
            canvasConfig: canvasConfig,
        });
        setBoard(newBoard);
        addListeners(newBoard.canvas);
        return function () {
            if (board) {
                board.removeBoard();
            }
        };
    }, [board]);
    useEffect(function () {
        if (!board || !canvasDrawingSettings)
            return;
        board.setDrawingSettings(canvasDrawingSettings);
    }, [canvasDrawingSettings, board]);
    useEffect(function () {
        if (!(board === null || board === void 0 ? void 0 : board.canvas) || !fileReaderInfo.currentPage) {
            return;
        }
        var json = getPageJSON({
            fileName: fileReaderInfo.file.name,
            pageNumber: fileReaderInfo.currentPageNumber,
        });
        if (json) {
            board.canvas.loadFromJSON(json);
        }
        else {
            board.openPage(fileReaderInfo.currentPage);
        }
    }, [fileReaderInfo.currentPage]);
    useEffect(function () {
        if (!board)
            return;
        setImage(backgroundImage);
    }, [board, backgroundImage]);
    function uploadImage(e) {
        var reader = new FileReader();
        var file = e.target.files[0];
        reader.addEventListener('load', function () {
            fabric.Image.fromURL(reader.result, function (img) {
                var _a;
                var imgWidth = img.width, imgHeight = img.height;
                var ratio = 1;
                if (imgWidth > imgHeight) {
                    ratio = maxWidth / imgWidth;
                }
                else {
                    ratio = maxHeight / imgHeight;
                }
                board.canvas.setWidth(imgWidth * ratio);
                board.canvas.setHeight(imgHeight * ratio);
                img.scaleToHeight(imgHeight * ratio);
                board.canvas.setBackgroundImage(img, board.canvas.renderAll.bind(board.canvas));
                (_a = board.canvas.get("backgroundImage")) === null || _a === void 0 ? void 0 : _a.set({ erasable: false });
                setFileReaderInfo(function (state) {
                    return __assign(__assign({}, state), { saveAsWidth: imgWidth, saveAsHeight: imgHeight, displayWidth: imgWidth * ratio, displayHeight: imgHeight * ratio });
                });
            });
        });
        reader.readAsDataURL(file);
    }
    function setImage(imageData) {
        var image = new Image();
        image.addEventListener('load', function (e) {
            fabric.Image.fromObject(image, function (img) {
                var _a;
                var imgWidth = img.width, imgHeight = img.height;
                var ratio = 1;
                if (imgWidth > imgHeight) {
                    ratio = maxWidth / imgWidth;
                }
                else {
                    ratio = maxHeight / imgHeight;
                }
                board.canvas.setWidth(img.width * ratio);
                board.canvas.setHeight(img.height * ratio);
                img.scaleToHeight(img.height * ratio);
                board.canvas.setBackgroundImage(img, board.canvas.renderAll.bind(board.canvas));
                (_a = board.canvas.get("backgroundImage")) === null || _a === void 0 ? void 0 : _a.set({ erasable: false });
                setFileReaderInfo(function (state) {
                    return __assign(__assign({}, state), { saveAsWidth: imgWidth, saveAsHeight: imgHeight, displayWidth: imgWidth * ratio, displayHeight: imgHeight * ratio });
                });
            });
        });
        image.src = imageData;
    }
    function addListeners(canvas) {
        canvas.on('after:render', function (e) {
            var data = getFullData(canvas);
            onCanvasRender(data, e, canvas);
        });
        canvas.on('zoom:change', function (data) {
            onZoom(data, null, canvas);
            setZoom(data.scale);
        });
        canvas.on('brush:change', function (data) {
            changeBrushWidth(data, canvas);
        });
        canvas.on('object:added', function (event) {
            onObjectAdded(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
        canvas.on('object:removed', function (event) {
            onObjectRemoved(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
        canvas.on('object:modified', function (event) {
            onObjectModified(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
    }
    function getFullData(canvas) {
        if (!canvas)
            return;
        return {
            settings: {
                contentJSON: canvas.toJSON(),
                viewportTransform: canvas.viewportTransform,
            },
            drawingSettings: canvasDrawingSettings,
            fileInfo: fileReaderInfo,
        };
    }
    function saveCanvasState() {
        var _a, _b;
        var newValue = __assign(__assign({}, canvasObjectsPerPage), (_a = {}, _a[fileReaderInfo.file.name] = __assign(__assign({}, canvasObjectsPerPage[fileReaderInfo.file.name]), (_b = {}, _b[fileReaderInfo.currentPageNumber] = board.canvas.toJSON(), _b)), _a));
        setCanvasObjectsPerPage(newValue);
        onSaveCanvasState(newValue);
    }
    function changeBrushWidth(e, canvas) {
        var intValue = parseInt(e.target.value);
        var newOptions = __assign(__assign(__assign({}, canvasDrawingSettings), { brushWidth: intValue }), (e.currentMode && { currentMode: e.currentMode }));
        setCanvasDrawingSettings(newOptions);
        if (canvas) {
            canvas.freeDrawingBrush.width = intValue;
            onOptionsChange(newOptions, e, canvas);
        }
        else {
            board.canvas.freeDrawingBrush.width = intValue;
            onOptionsChange(newOptions, e, board.canvas);
        }
    }
    function changeMode(mode, e) {
        if (canvasDrawingSettings.currentMode === mode)
            return;
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { currentMode: mode });
        setCanvasDrawingSettings(newOptions);
        board.setDrawingMode(mode);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function changeCurrentColor(color, e) {
        board.canvas.freeDrawingBrush.color = color;
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { currentColor: color });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function changeFill(e) {
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { fill: !canvasDrawingSettings.fill });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function handleSaveCanvasAsImage() {
        board.resetZoom();
        board.resetZoom();
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.saveAsWidth / fileReaderInfo.displayWidth });
        board.canvas.setWidth(fileReaderInfo.saveAsWidth);
        board.canvas.setHeight(fileReaderInfo.saveAsHeight);
        board.canvas.renderAll();
        canvasRef.current.toBlob(function (blob) {
            saveAs(blob, "".concat(fileReaderInfo.file.name).concat(fileReaderInfo.currentPage ? '_page-' : '', ".png"));
            onSaveCanvasAsImage(blob, null, board.canvas);
        });
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.displayWidth / fileReaderInfo.saveAsWidth });
        board.canvas.setWidth(fileReaderInfo.displayWidth);
        board.canvas.setHeight(fileReaderInfo.displayHeight);
        board.resetZoom();
        board.canvas.renderAll();
    }
    function handleSaveCanvasAsImageWithoutBackgroundImage() {
        var backgroundImage = board.canvas.backgroundImage;
        board.resetZoom();
        board.resetZoom();
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.saveAsWidth / fileReaderInfo.displayWidth });
        board.canvas.setWidth(fileReaderInfo.saveAsWidth);
        board.canvas.setHeight(fileReaderInfo.saveAsHeight);
        board.canvas.setBackgroundImage(null);
        board.canvas.renderAll();
        canvasRef.current.toBlob(function (blob) {
            saveAs(blob, "".concat(fileReaderInfo.file.name).concat(fileReaderInfo.currentPage ? '_page-' : '', ".png"));
            onSaveCanvasAsImage(blob, null, board.canvas);
        });
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.displayWidth / fileReaderInfo.saveAsWidth });
        board.canvas.setWidth(fileReaderInfo.displayWidth);
        board.canvas.setHeight(fileReaderInfo.displayHeight);
        board.resetZoom();
        board.canvas.setBackgroundImage(backgroundImage);
        board.canvas.renderAll();
    }
    function handleExportBlob() {
        board.resetZoom();
        board.resetZoom();
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.saveAsWidth / fileReaderInfo.displayWidth });
        board.canvas.setWidth(fileReaderInfo.saveAsWidth);
        board.canvas.setHeight(fileReaderInfo.saveAsHeight);
        board.canvas.renderAll();
        var dataURL = canvasRef.current.toDataURL("image/png");
        onExportBlob(dataURL, null, board.canvas);
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.displayWidth / fileReaderInfo.saveAsWidth });
        board.canvas.setWidth(fileReaderInfo.displayWidth);
        board.canvas.setHeight(fileReaderInfo.displayHeight);
        board.resetZoom();
        board.canvas.renderAll();
    }
    function handleExportBlobWithoutBackgroundImage() {
        var backgroundImage = board.canvas.backgroundImage;
        board.resetZoom();
        board.resetZoom();
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.saveAsWidth / fileReaderInfo.displayWidth });
        board.canvas.setWidth(fileReaderInfo.saveAsWidth);
        board.canvas.setHeight(fileReaderInfo.saveAsHeight);
        board.canvas.setBackgroundImage(null);
        board.canvas.renderAll();
        var dataURL = canvasRef.current.toDataURL("image/png");
        onExportBlob(dataURL, null, board.canvas);
        board.changeZoom({ point: { x: 0, y: 0 }, scale: fileReaderInfo.displayWidth / fileReaderInfo.saveAsWidth });
        board.canvas.setWidth(fileReaderInfo.displayWidth);
        board.canvas.setHeight(fileReaderInfo.displayHeight);
        board.resetZoom();
        board.canvas.setBackgroundImage(backgroundImage);
        board.canvas.renderAll();
    }
    function onFileChange(event) {
        if (!event.target.files[0])
            return;
        if (event.target.files[0].type.includes('image/')) {
            uploadImage(event);
            onImageUploaded(event.target.files[0], event, board.canvas);
            board.resetZoom();
        }
        else if (event.target.files[0].type.includes('pdf')) {
            saveCanvasState();
            board.clearCanvas();
            updateFileReaderInfo({ file: event.target.files[0], currentPageNumber: 1 });
            onPDFUploaded(event.target.files[0], event, board.canvas);
        }
        event.target.value = '';
    }
    function sketchSizeChange(e) {
        console.log(e.target.value);
        var values = e.target.value.split('x');
        var width = values[0], height = values[1];
        var ratio = 1;
        if (width > height) {
            ratio = maxWidth / width;
        }
        else {
            ratio = maxHeight / height;
        }
        board.resetZoom();
        board.canvas.setWidth(width * ratio);
        board.canvas.setHeight(height * ratio);
        board.canvas.renderAll();
    }
    function getPageJSON(_a) {
        var fileName = _a.fileName, pageNumber = _a.pageNumber;
        if (canvasObjectsPerPage[fileName] && canvasObjectsPerPage[fileName][pageNumber]) {
            return canvasObjectsPerPage[fileName][pageNumber];
        }
        else {
            return null;
        }
    }
    function updateFileReaderInfo(data) {
        var newFileData = __assign(__assign({}, fileReaderInfo), data);
        setFileReaderInfo(newFileData);
        onPDFUpdated(newFileData, null, board.canvas);
    }
    var handlePageChange = function (page) {
        saveCanvasState();
        board.clearCanvas(board.canvas);
        setFileReaderInfo(__assign(__assign({}, fileReaderInfo), { currentPageNumber: page }));
        onPageChange(__assign(__assign({}, fileReaderInfo), { currentPageNumber: page }), null, board.canvas);
    };
    var handleZoomIn = function () {
        board.changeZoom({ scale: 1.1 });
    };
    var handleZoomOut = function () {
        board.changeZoom({ scale: 0.9 });
    };
    var handleResetZoom = function () {
        board.resetZoom(1);
    };
    var getColorButtons = function (colors) {
        return colors.map(function (color) { return (React.createElement(ToolbarItemS, { key: color },
            React.createElement(ColorButtonS, { color: color, onClick: function (e) { return changeCurrentColor(color, e); } }))); });
    };
    var getControls = function () {
        var _a;
        var modeButtons = (_a = {},
            _a[modes.PENCIL] = { icon: PencilIcon, name: 'Pencil' },
            _a[modes.MOVE] = { icon: SelectIcon, name: 'Move' },
            _a[modes.ERASERDRAW] = { icon: EraserIcon, name: 'EraserDraw' },
            _a);
        return Object.keys(modeButtons).map(function (buttonKey) {
            if (!enabledControls[buttonKey])
                return;
            var btn = modeButtons[buttonKey];
            return (React.createElement(ButtonS, { key: buttonKey, type: "button", className: "".concat(canvasDrawingSettings.currentMode === buttonKey ? 'selected' : ''), onClick: function (e) { return changeMode(buttonKey, e); } },
                React.createElement("img", { src: btn.icon, alt: btn.name })));
        });
    };
    return (React.createElement(PaintPanelS, { ref: whiteboardRef, width: "".concat(maxWidth, "px"), height: "".concat(maxHeight, "px") },
        React.createElement(ToolbarHolderS, null,
            React.createElement(ColorBarS, null,
                !!enabledControls.COLOR_PICKER && (React.createElement(ToolbarItemS, null,
                    React.createElement(ColorPicker, { size: 28, color: canvasDrawingSettings.currentColor, onChange: changeCurrentColor }))),
                !!enabledControls.BRUSH && (React.createElement(ToolbarItemS, null,
                    React.createElement(RangeInputS, { ref: rangeInput, type: "range", min: 1, max: 30, step: 1, thumbColor: canvasDrawingSettings.currentColor, value: canvasDrawingSettings.brushWidth, onChange: changeBrushWidth }))),
                !!enabledControls.DEFAULT_COLORS && (React.createElement(React.Fragment, null, getColorButtons(['#6161ff', '#ff4f4f', '#3fd18d', '#ec70ff', '#000000']))),
                !!enabledControls.FILL && (React.createElement(ButtonS, { type: "button", className: canvasDrawingSettings.fill ? 'selected' : '', onClick: changeFill },
                    React.createElement("img", { src: FillIcon, alt: "Delete" })))),
            React.createElement(ToolbarS, null,
                mode === 'sketch' && sizes && (React.createElement(SelectS, { onChange: sketchSizeChange }, sizes.map(function (item) { return (React.createElement(OptionS, { key: item }, item)); }))),
                getControls(),
                !!enabledControls.CLEAR && (React.createElement(ButtonS, { type: "button", onClick: function () { return board.clearCanvas(); } },
                    React.createElement("img", { src: DeleteIcon, alt: "Delete" }))),
                React.createElement(SeparatorS, null),
                !!enabledControls.FILES && (React.createElement(ToolbarItemS, null,
                    React.createElement("input", { ref: uploadPdfRef, hidden: true, accept: "image/*", type: "file", onChange: onFileChange }),
                    React.createElement(ButtonS, { onClick: function () { return uploadPdfRef.current.click(); } },
                        React.createElement("img", { src: UploadIcon, alt: "Delete" })))),
                !!enabledControls.SAVE_AS_IMAGE && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleSaveCanvasAsImage },
                        React.createElement("img", { src: DownloadIcon, alt: "Download" })))),
                !!enabledControls.SAVE_AS_IMAGE_WITHOUT_BACKGROUNDIMAGE && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleSaveCanvasAsImageWithoutBackgroundImage },
                        React.createElement("img", { src: DownloadIcon, alt: "Download" })))),
                !!enabledControls.EXPORT_IMAGE && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleExportBlob },
                        React.createElement("img", { src: DownloadIcon, alt: "Download" })))),
                !!enabledControls.EXPORT_IMAGE_WITHOUT_BACKGROUNDIMAGE && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleExportBlobWithoutBackgroundImage },
                        React.createElement("img", { src: DownloadIcon, alt: "Download" }))))),
            React.createElement(ZoomBarS, null,
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleZoomIn, title: "Zoom In" },
                        React.createElement("img", { src: ZoomInIcon, alt: "Zoom In" })))),
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleResetZoom, title: "Reset Zoom" },
                        React.createElement("span", { style: { fontSize: '11px' } },
                            Math.floor(zoom * 100),
                            "%")))),
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleZoomOut, title: "Zoom Out" },
                        React.createElement("img", { src: ZoomOutIcon, alt: "Zoom Out" })))))),
        React.createElement(DrawPanelS, null,
            React.createElement("canvas", { ref: canvasRef, id: "canvas" })),
        React.createElement(PDFWrapperS, null,
            React.createElement(PdfReader, { fileReaderInfo: fileReaderInfo, onPageChange: handlePageChange, updateFileReaderInfo: updateFileReaderInfo }))));
};
Paintboard.propTypes = {
    aspectRatio: PropTypes.number,
};
export default Paintboard;
//# sourceMappingURL=Paintboard.js.map