import { __assign } from "tslib";
import React, { useState, useRef, useEffect, useMemo } from 'react';
import PropTypes from 'prop-types';
import { WhiteBoardS, ButtonS, ToolbarS, ZoomBarS, ColorBarS, ToolbarItemS, RangeInputS, ColorButtonS, SeparatorS, ToolbarHolderS, PDFWrapperS, } from './Whiteboard.styled';
import { fabric } from 'fabric-all-modules';
import { PdfReader } from '../PdfReader';
import { saveAs } from 'file-saver';
import { Board, modes } from './Board.Class.js';
import { ColorPicker } from '../ColorPicker';
import SelectIcon from '../images/cross.svg';
import EraserIcon from '../images/eraser.svg';
import TextIcon from '../images/text.svg';
import RectangleIcon from '../images/rectangle.svg';
import LineIcon from '../images/line.svg';
import EllipseIcon from '../images/ellipse.svg';
import TriangleIcon from '../images/triangle.svg';
import PencilIcon from '../images/pencil.svg';
import DeleteIcon from '../images/delete.svg';
import ZoomInIcon from '../images/zoom-in.svg';
import ZoomOutIcon from '../images/zoom-out.svg';
import DownloadIcon from '../images/download.svg';
import UploadIcon from '../images/add-photo.svg';
import FillIcon from '../images/color-fill.svg';
var initFileInfo = {
    file: { name: 'whiteboard' },
    totalPages: 1,
    currentPageNumber: 0,
    currentPage: '',
};
var initDrawingSettings = {
    brushWidth: 5,
    currentMode: 'PENCIL',
    currentColor: '#000000',
    brushWidth: 5,
    fill: false,
};
var initSettings = {
    zoom: 1,
    contentJSON: null,
};
var Whiteboard = function (_a) {
    var controls = _a.controls, settings = _a.settings, drawingSettings = _a.drawingSettings, fileInfo = _a.fileInfo, _b = _a.onObjectAdded, onObjectAdded = _b === void 0 ? function (data, event, canvas) { } : _b, _c = _a.onObjectRemoved, onObjectRemoved = _c === void 0 ? function (data, event, canvas) { } : _c, _d = _a.onObjectModified, onObjectModified = _d === void 0 ? function (data, event, canvas) { } : _d, _e = _a.onCanvasRender, onCanvasRender = _e === void 0 ? function (data, event, canvas) { } : _e, _f = _a.onCanvasChange, onCanvasChange = _f === void 0 ? function (data, event, canvas) { } : _f, _g = _a.onZoom, onZoom = _g === void 0 ? function (data, event, canvas) { } : _g, _h = _a.onImageUploaded, onImageUploaded = _h === void 0 ? function (data, event, canvas) { } : _h, _j = _a.onPDFUploaded, onPDFUploaded = _j === void 0 ? function (data, event, canvas) { } : _j, _k = _a.onPDFUpdated, onPDFUpdated = _k === void 0 ? function (data, event, canvas) { } : _k, _l = _a.onPageChange, onPageChange = _l === void 0 ? function (data, event, canvas) { } : _l, _m = _a.onOptionsChange, onOptionsChange = _m === void 0 ? function (data, event, canvas) { } : _m, _o = _a.onSaveCanvasAsImage, onSaveCanvasAsImage = _o === void 0 ? function (data, event, canvas) { } : _o, _p = _a.onConfigChange, onConfigChange = _p === void 0 ? function (data, event, canvas) { } : _p, _q = _a.onSaveCanvasState, onSaveCanvasState = _q === void 0 ? function (data, event, canvas) { } : _q;
    var _r = useState(), board = _r[0], setBoard = _r[1];
    var _s = useState({}), canvasObjectsPerPage = _s[0], setCanvasObjectsPerPage = _s[1];
    var _t = useState(__assign(__assign({}, initDrawingSettings), drawingSettings)), canvasDrawingSettings = _t[0], setCanvasDrawingSettings = _t[1];
    var canvasConfig = __assign(__assign({}, initSettings), settings);
    var _u = useState(canvasConfig.zoom), zoom = _u[0], setZoom = _u[1];
    var _v = useState(__assign(__assign({}, initFileInfo), fileInfo)), fileReaderInfo = _v[0], setFileReaderInfo = _v[1];
    var canvasRef = useRef(null);
    var whiteboardRef = useRef(null);
    var uploadPdfRef = useRef(null);
    var enabledControls = useMemo(function () {
        var _a;
        return __assign((_a = {}, _a[modes.PENCIL] = true, _a[modes.LINE] = true, _a[modes.RECTANGLE] = true, _a[modes.ELLIPSE] = true, _a[modes.TRIANGLE] = true, _a[modes.TEXT] = true, _a[modes.SELECT] = true, _a[modes.ERASER] = true, _a.CLEAR = true, _a.FILL = true, _a.BRUSH = true, _a.COLOR_PICKER = true, _a.DEFAULT_COLORS = true, _a.FILES = true, _a.SAVE_AS_IMAGE = true, _a.ZOOM = true, _a), controls);
    }, [controls]);
    useEffect(function () {
        setCanvasDrawingSettings(__assign(__assign({}, canvasDrawingSettings), drawingSettings));
    }, [drawingSettings]);
    useEffect(function () {
        if (!board || !canvasConfig)
            return;
        board.setCanvasConfig(canvasConfig);
    }, [settings]);
    useEffect(function () {
        setFileReaderInfo(__assign(__assign({}, fileReaderInfo), fileInfo));
    }, [fileInfo]);
    useEffect(function () {
        if (board) {
            return;
        }
        var newBoard = new Board({
            drawingSettings: canvasDrawingSettings,
            canvasConfig: canvasConfig,
        });
        setBoard(newBoard);
        addListeners(newBoard.canvas);
        return function () {
            if (board) {
                board.removeBoard();
            }
        };
    }, [board]);
    useEffect(function () {
        if (!board || !canvasDrawingSettings)
            return;
        board.setDrawingSettings(canvasDrawingSettings);
    }, [canvasDrawingSettings, board]);
    useEffect(function () {
        if (!(board === null || board === void 0 ? void 0 : board.canvas) || !fileReaderInfo.currentPage) {
            return;
        }
        var json = getPageJSON({
            fileName: fileReaderInfo.file.name,
            pageNumber: fileReaderInfo.currentPageNumber,
        });
        if (json) {
            board.canvas.loadFromJSON(json);
        }
        else {
            board.openPage(fileReaderInfo.currentPage);
        }
    }, [fileReaderInfo.currentPage]);
    function uploadImage(e) {
        var reader = new FileReader();
        var file = e.target.files[0];
        reader.addEventListener('load', function () {
            fabric.Image.fromURL(reader.result, function (img) {
                img.scaleToHeight(board.canvas.height);
                board.canvas.add(img);
            });
        });
        reader.readAsDataURL(file);
    }
    function addListeners(canvas) {
        canvas.on('after:render', function (e) {
            var data = getFullData(canvas);
            onCanvasRender(data, e, canvas);
        });
        canvas.on('zoom:change', function (data) {
            onZoom(data, null, canvas);
            setZoom(data.scale);
        });
        canvas.on('object:added', function (event) {
            onObjectAdded(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
        canvas.on('object:removed', function (event) {
            onObjectRemoved(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
        canvas.on('object:modified', function (event) {
            onObjectModified(event.target.toJSON(), event, canvas);
            onCanvasChange(event.target.toJSON(), event, canvas);
        });
    }
    function getFullData(canvas) {
        if (!canvas)
            return;
        return {
            settings: {
                contentJSON: canvas.toJSON(),
                viewportTransform: canvas.viewportTransform,
            },
            drawingSettings: canvasDrawingSettings,
            fileInfo: fileReaderInfo,
        };
    }
    function saveCanvasState() {
        var _a, _b;
        var newValue = __assign(__assign({}, canvasObjectsPerPage), (_a = {}, _a[fileReaderInfo.file.name] = __assign(__assign({}, canvasObjectsPerPage[fileReaderInfo.file.name]), (_b = {}, _b[fileReaderInfo.currentPageNumber] = board.canvas.toJSON(), _b)), _a));
        setCanvasObjectsPerPage(newValue);
        onSaveCanvasState(newValue);
    }
    function changeBrushWidth(e) {
        var intValue = parseInt(e.target.value);
        board.canvas.freeDrawingBrush.width = intValue;
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { brushWidth: intValue });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function changeMode(mode, e) {
        if (canvasDrawingSettings.currentMode === mode)
            return;
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { currentMode: mode });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function changeCurrentColor(color, e) {
        board.canvas.freeDrawingBrush.color = color;
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { currentColor: color });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function changeFill(e) {
        var newOptions = __assign(__assign({}, canvasDrawingSettings), { fill: !canvasDrawingSettings.fill });
        setCanvasDrawingSettings(newOptions);
        onOptionsChange(newOptions, e, board.canvas);
    }
    function handleSaveCanvasAsImage() {
        canvasRef.current.toBlob(function (blob) {
            saveAs(blob, "".concat(fileReaderInfo.file.name).concat(fileReaderInfo.currentPage ? '_page-' : '', ".png"));
            onSaveCanvasAsImage(blob, null, board.canvas);
        });
    }
    function onFileChange(event) {
        if (!event.target.files[0])
            return;
        if (event.target.files[0].type.includes('image/')) {
            uploadImage(event);
            onImageUploaded(event.target.files[0], event, board.canvas);
        }
        else if (event.target.files[0].type.includes('pdf')) {
            saveCanvasState();
            board.clearCanvas();
            updateFileReaderInfo({ file: event.target.files[0], currentPageNumber: 1 });
            onPDFUploaded(event.target.files[0], event, board.canvas);
        }
    }
    function getPageJSON(_a) {
        var fileName = _a.fileName, pageNumber = _a.pageNumber;
        if (canvasObjectsPerPage[fileName] && canvasObjectsPerPage[fileName][pageNumber]) {
            return canvasObjectsPerPage[fileName][pageNumber];
        }
        else {
            return null;
        }
    }
    function updateFileReaderInfo(data) {
        var newFileData = __assign(__assign({}, fileReaderInfo), data);
        setFileReaderInfo(newFileData);
        onPDFUpdated(newFileData, null, board.canvas);
    }
    var handlePageChange = function (page) {
        saveCanvasState();
        board.clearCanvas(board.canvas);
        setFileReaderInfo(__assign(__assign({}, fileReaderInfo), { currentPageNumber: page }));
        onPageChange(__assign(__assign({}, fileReaderInfo), { currentPageNumber: page }), null, board.canvas);
    };
    var handleZoomIn = function () {
        board.changeZoom({ scale: 1.1 });
    };
    var handleZoomOut = function () {
        board.changeZoom({ scale: 0.9 });
    };
    var handleResetZoom = function () {
        board.resetZoom(1);
    };
    var getColorButtons = function (colors) {
        return colors.map(function (color) { return (React.createElement(ToolbarItemS, { key: color },
            React.createElement(ColorButtonS, { color: color, onClick: function (e) { return changeCurrentColor(color, e); } }))); });
    };
    var getControls = function () {
        var _a;
        var modeButtons = (_a = {},
            _a[modes.PENCIL] = { icon: PencilIcon, name: 'Pencil' },
            _a[modes.LINE] = { icon: LineIcon, name: 'Line' },
            _a[modes.RECTANGLE] = { icon: RectangleIcon, name: 'Rectangle' },
            _a[modes.ELLIPSE] = { icon: EllipseIcon, name: 'Ellipse' },
            _a[modes.TRIANGLE] = { icon: TriangleIcon, name: 'Triangle' },
            _a[modes.TEXT] = { icon: TextIcon, name: 'Text' },
            _a[modes.SELECT] = { icon: SelectIcon, name: 'Select' },
            _a[modes.ERASER] = { icon: EraserIcon, name: 'Eraser' },
            _a);
        return Object.keys(modeButtons).map(function (buttonKey) {
            if (!enabledControls[buttonKey])
                return;
            var btn = modeButtons[buttonKey];
            return (React.createElement(ButtonS, { key: buttonKey, type: "button", className: "".concat(canvasDrawingSettings.currentMode === buttonKey ? 'selected' : ''), onClick: function (e) { return changeMode(buttonKey, e); } },
                React.createElement("img", { src: btn.icon, alt: btn.name })));
        });
    };
    return (React.createElement(WhiteBoardS, { ref: whiteboardRef },
        React.createElement(ToolbarHolderS, null,
            React.createElement(ColorBarS, null,
                !!enabledControls.COLOR_PICKER && (React.createElement(ToolbarItemS, null,
                    React.createElement(ColorPicker, { size: 28, color: canvasDrawingSettings.currentColor, onChange: changeCurrentColor }))),
                !!enabledControls.BRUSH && (React.createElement(ToolbarItemS, null,
                    React.createElement(RangeInputS, { type: "range", min: 1, max: 30, step: 1, thumbColor: canvasDrawingSettings.currentColor, value: canvasDrawingSettings.brushWidth, onChange: changeBrushWidth }))),
                !!enabledControls.DEFAULT_COLORS && (React.createElement(React.Fragment, null, getColorButtons(['#6161ff', '#ff4f4f', '#3fd18d', '#ec70ff', '#000000']))),
                !!enabledControls.FILL && (React.createElement(ButtonS, { type: "button", className: canvasDrawingSettings.fill ? 'selected' : '', onClick: changeFill },
                    React.createElement("img", { src: FillIcon, alt: "Delete" })))),
            React.createElement(ToolbarS, null,
                getControls(),
                !!enabledControls.CLEAR && (React.createElement(ButtonS, { type: "button", onClick: function () { return board.clearCanvas(); } },
                    React.createElement("img", { src: DeleteIcon, alt: "Delete" }))),
                React.createElement(SeparatorS, null),
                !!enabledControls.FILES && (React.createElement(ToolbarItemS, null,
                    React.createElement("input", { ref: uploadPdfRef, hidden: true, accept: "image/*,.pdf", type: "file", onChange: onFileChange }),
                    React.createElement(ButtonS, { onClick: function () { return uploadPdfRef.current.click(); } },
                        React.createElement("img", { src: UploadIcon, alt: "Delete" })))),
                !!enabledControls.SAVE_AS_IMAGE && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleSaveCanvasAsImage },
                        React.createElement("img", { src: DownloadIcon, alt: "Download" }))))),
            React.createElement(ZoomBarS, null,
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleZoomIn, title: "Zoom In" },
                        React.createElement("img", { src: ZoomInIcon, alt: "Zoom In" })))),
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleResetZoom, title: "Reset Zoom" },
                        React.createElement("span", { style: { fontSize: '11px' } },
                            Math.floor(zoom * 100),
                            "%")))),
                !!enabledControls.ZOOM && (React.createElement(ToolbarItemS, null,
                    React.createElement(ButtonS, { onClick: handleZoomOut, title: "Zoom Out" },
                        React.createElement("img", { src: ZoomOutIcon, alt: "Zoom Out" })))))),
        React.createElement("canvas", { ref: canvasRef, id: "canvas" }),
        React.createElement(PDFWrapperS, null,
            React.createElement(PdfReader, { fileReaderInfo: fileReaderInfo, onPageChange: handlePageChange, updateFileReaderInfo: updateFileReaderInfo }))));
};
Whiteboard.propTypes = {
    aspectRatio: PropTypes.number,
};
export default Whiteboard;
//# sourceMappingURL=Whiteboard.js.map